# -*- coding: utf-8 -*-

from flask import Blueprint, current_app, jsonify, request
from .model import PersonalAccessToken

bp = Blueprint('personal_access_token.api', __name__)

@bp.before_request
def before_request():
    return bp.app.call_before_request_funcs()

@bp.route('/tokens')
def get_tokens():
    tokens = PersonalAccessToken.query.filter_by(user_id=bp.app.current_user_id).all()
    return ok([token.to_dict() for token in tokens])


@bp.route('/tokens', methods=['POST'])
def create_token():
    user_id = bp.app.current_user_id
    data = _get_form_data()
    if not data:
        return bad_request()
    description = data['description']
    scopes = data['scopes']
    token = PersonalAccessToken.add(user_id, description, scopes)
    return ok(token.to_full_dict())

@bp.route('/tokens/<int:id>')
def get_token(id):
    token = PersonalAccessToken.query.get(id)
    error = _validate_token(token)
    if error:
        return error
    data = token.to_dict()
    return ok(data)

@bp.route('/tokens/<int:id>', methods=['DELETE'])
def delete_token(id):
    token = PersonalAccessToken.query.get(id)
    error = _validate_token(token)
    if error:
        return error
    token.delete()
    return ok()

@bp.route('/tokens/<int:id>', methods=['PUT'])
def update_token(id):
    token = PersonalAccessToken.query.get(id)
    error = _validate_token(token)
    if error:
        return error
    data = _get_form_data()
    if not data:
        return bad_request()
    description = data['description']
    scopes = data['scopes']
    token = token.update(description, scopes)
    return ok(token.to_dict())



def _get_form_data():
    data = request.get_json()
    if not data:
        return
    if 'description' not in data:
        return
    scopes = data.get('scopes')
    scopes = scopes or []
    if not isinstance(scopes, list):
        return
    for scope in scopes:
        if not isinstance(scope, str) and scope:
            return
    description = data['description']
    return dict(scopes=scopes, description=description)

def _validate_token(token):
    if not token:
        return not_found()
    if token.user_id != bp.app.current_user_id:
        return forbidden()

def bad_request():
    return jsonify(), 400

def not_found():
    return jsonify(), 404

def forbidden():
    return jsonify(), 403

def ok(data=None):
    return jsonify(data=data)



# -*- coding: utf-8 -*-

from flask import Blueprint

bp = Blueprint('personal_access_token.web', __name__)

@bp.before_request
def before_request():
    bp.app.call_before_request_funcs()

@bp.route('/')
def index():
    return render_template('personal_access_token/index.html')



# -*- coding: utf-8 -*-

from flask import Blueprint, render_template, current_app, url_for, redirect, request, flash

bp = Blueprint(
    'personal-access-token-admin',
    __name__,
    template_folder='templates',
    static_folder='static'
)

@bp.before_request
def before_request():
    return bp.app.call_before_request_funcs()

@bp.route('/')
def index():
    render_data = {
        'base_api_url': current_app.config.get('PERSONAL_ACCESS_TOKEN_ADMIN_API_PREFIX'),
        'base_web_url': current_app.config.get('PERSONAL_ACCESS_TOKEN_ADMIN_PREFIX'),
        'debug': current_app.config.get('DEBUG'),
    }
    return render_template('/personal_access_token/index.html', **render_data)



# -*- coding: utf-8 -*-

from flask import request
from .core import db
from .model import PersonalAccessToken

class PersonalAccessTokenManager(object):

    def __init__(self, app=None):
        self.app = app
        self.before_request_funcs = []
        self.current_user_id = None
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        db.app = app
        db.init_app(app)

        app.config.setdefault('PERSONAL_ACCESS_TOKEN_ADMIN_API_PREFIX', '/personal_access_token/api')
        app.config.setdefault('PERSONAL_ACCESS_TOKEN_ADMIN_PREFIX', '/personal_access_token')

        from .api import bp as api_bp
        api_bp.app = self
        app.register_blueprint(api_bp, url_prefix='/personal_access_token/api')

        from .admin import bp as admin_bp
        admin_bp.app = self
        app.register_blueprint(admin_bp, url_prefix='/personal_access_token/')

    def create_all(self):
        db.create_all()

    def call_before_request_funcs(self):
        for func in self.before_request_funcs:
            func()

    def before_request(self, f):
        self.before_request_funcs.append(f)
        return f

    def user_loader(self, f):
        self.user_loader_callback = f
        return f

    def load_user(self, user_id):
        if not self.user_loader_callback:
            raise NotImplementedError('You must implement `user_loader`.')
        return self.user_loader_callback(user_id)

    def load_user_by_token(self, token):
        token = PersonalAccessToken.query.filter_by(token=token).first()
        if not token:
            return
        token.use()
        return self.load_user(token.user_id)

    def load_user_from_request(self):
        headers = request.headers
        authorization = headers.get('Authorization')
        if not authorization or not authorization.startswith('Bearer '):
            return
        token = authorization[len('Bearer '):]
        return self.load_user_by_token(token)



# -*- coding: utf-8 -*-

from datetime import datetime
from werkzeug.security import generate_password_hash
from .core import db

class PersonalAccessToken(db.Model):

    __tablename__ = 'personal_access_token'

    __table_args__ = (
        db.UniqueConstraint('token', name='ux_token'),
    )

    id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(db.Integer, nullable=False)
    description = db.Column(db.String(64), nullable=False)
    salt = db.Column(db.String(8), nullable=False)
    token = db.Column(db.String(40), nullable=False)
    scopes = db.Column(db.Text)
    created_at = db.Column(db.DateTime, default=datetime.utcnow)
    last_used_at = db.Column(db.DateTime)

    def to_dict(self):
        return dict(
            id=self.id,
            user_id=self.user_id,
            description=self.description,
            scopes=[] if not self.scopes else self.scopes.split(','),
            token=self.token,
            last_used_at=self.last_used_at and self.last_used_at.isoformat(),
        )

    def to_full_dict(self):
        data = self.to_dict()
        data['token'] = self.token
        return data

    def delete(self):
        db.session.delete(self)
        db.session.commit()

    @classmethod
    def generate_token(cls, user_id):
        data = generate_password_hash(str(user_id))
        method, salt, token = data.split('$')
        return dict(
            token=token,
            salt=salt
        )

    def reset_token(self):
        data = self.generate_token(self.id)
        self.token = data['token']
        self.salt = data['salt']
        db.session.add(self)
        db.session.commit()
        return self

    def update(self, description, scopes):
        self.description = description
        self.scopes = ','.join(scopes)
        db.session.add(self)
        db.session.commit()
        return self

    @classmethod
    def add(cls, user_id, description, scopes=[]):
        token_data = cls.generate_token(user_id)
        obj = cls(
            user_id=user_id,
            description=description,
            scopes=','.join(scopes),
            salt=token_data['salt'],
            token=token_data['token'],
        )
        db.session.add(obj)
        db.session.commit()
        return obj

    def use(self):
        self.last_used_at = datetime.utcnow()
        db.session.add(self)
        db.session.commit()
